/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2018 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_main.c
* Version      : CodeGenerator for RL78/G13 V2.05.01.04 [18 May 2018]
* Device(s)    : R5F100GG
* Tool-Chain   : CCRL
* Description  : This file implements main function.
* Creation Date: 
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_cgc.h"
#include "r_cg_port.h"
#include "r_cg_serial.h"
/* Start user code for include. Do not edit comment generated here */
#include <stdio.h>
#include "b_uart.h"
/* End user code. Do not edit comment generated here */
#include "r_cg_userdefine.h"

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
/* Start user code for global. Do not edit comment generated here */
Uart uart_debug;

uint8_t status = 0U;

extern R_VOID_CALLBACK_P r_uart1_sendend_callback_p;
extern R_VOID_CALLBACK_P r_uart1_receiveend_callback_p;
extern R_UINT8T_CALLBACK_P r_uart1_error_callback_p;
extern R_UINT16T_CALLBACK_P r_uart1_overrun_callback_p;

extern R_VOID_CALLBACK_P r_csi21_receiveend_callback_p;
extern R_UINT8T_CALLBACK_P r_csi21_error_callback_p;

/**
 * @brief UARTのデータ送信完了割り込みから呼ばれるコールバックの実装
 */
static void uart1_sendend_cb(void)
{
    uart_debug.write_complete(&uart_debug);
    return;
}

/**
 * @brief UARTのデータ受信完了割り込みから呼ばれるコールバックの実装
 */
static void uart1_receiveend_cb(void)
{
    uart_debug.read_complete(&uart_debug);
    return;
}

/**
 * @brief UARTのエラー発生割り込みから呼ばれるコールバックの実装
 */
static void uart1_error_cb(uint8_t err_type)
{
    uart_debug.error_callback(&uart_debug, err_type);
    return;
}

/**
 * @brief UARTのオーバーラン発生割り込みから呼ばれるコールバックの実装
 */
static void uart1_overrun_cb(uint16_t rx_data)
{
    uart_debug.overrun_callback(&uart_debug, rx_data);
    return;
}

/**
 * @brief CSI21の送受信完了割り込みから呼ばれるコールバックの実装
 */
static void csi21_receiveend_cb(void)
{
    status = 1U;
    return;
}

/**
 * @brief CSI21のエラー割り込みから呼ばれるコールバックの実装
 */
static void csi21_error_cb(void)
{
    status = 2U;
    return;
}

/**
 * printfの内容をUARTに出力するためにputcharを定義しなおした関数
 *
 * @param c 出力する文字
 * @return 正常に完了した時： 引数, エラー発生時： EOF
 */
int putchar(int c)
{
    if (uart_debug.putchar(&uart_debug, c) == c)
    {
        return (c);
    }
    else
    {
        return (EOF);
    }
}
/* End user code. Do not edit comment generated here */
void R_MAIN_UserInit(void);

/***********************************************************************************************************************
* Function Name: main
* Description  : This function implements main function.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void main(void)
{
    R_MAIN_UserInit();
    /* Start user code. Do not edit comment generated here */
    {
        uint8_t tx_buf[10U];
        uint8_t rx_buf[10U];
        uint8_t tx_num;

        tx_buf[0] = 0x03;
        tx_buf[1] = 0x00;
        tx_buf[2] = 0x00;
        tx_buf[3] = 0x00;

        tx_num = 4U;

        P7_bit.no5 = 0U;  // CSをLに

        R_CSI21_Send_Receive(tx_buf, tx_num, rx_buf);

        while (1)
        {
            if (status == 1U)
            {
                break;
            }
            else if (status == 2U)
            {
                break;
            }
            else
            {
                NOP();
            }
            HALT();
        }

        P7_bit.no5 = 1U;   // CSをHに

        printf("%d\n", status);

		while (1)
		{
			NOP();
		}
    }
    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: R_MAIN_UserInit
* Description  : This function adds user code before implementing main function.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_MAIN_UserInit(void)
{
    /* Start user code. Do not edit comment generated here */
    EI();

    // UARTのインスタンスを作る
    Uart_Conf uart_debug_conf =
    { .timeout = 0xFFFFU, .func_receive = R_UART1_Receive, .func_send = R_UART1_Send };
    uart_debug = new_Uart(uart_debug_conf);

    // UARTのコールバックを設定する
    r_uart1_receiveend_callback_p = uart1_receiveend_cb;
    r_uart1_sendend_callback_p = uart1_sendend_cb;
    r_uart1_error_callback_p = uart1_error_cb;
    r_uart1_overrun_callback_p = uart1_overrun_cb;

    R_UART1_Start();

    // CSIのコールバックを設定する
    r_csi21_receiveend_callback_p = csi21_receiveend_cb;
    r_csi21_error_callback_p = csi21_error_cb;

    R_CSI21_Start();

    /* End user code. Do not edit comment generated here */
}

/* Start user code for adding. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
